@@ -18,6 +18,9 @@ import Sound.Tidal.SuperCollider as C
 import Sound.Tidal.Params as C
 import Sound.Tidal.Transition as C
 import Sound.Tidal.Version as C
-import qualified Sound.Tidal.Scales as Scales
-import qualified Sound.Tidal.Chords as Chords
+import Sound.Tidal.Chords as C (chord)
+import Sound.Tidal.Scales as C (scaleP)
+
+import qualified Sound.Tidal.Scales as C.Scales
+import qualified Sound.Tidal.Chords as C.Chords
 
