@@ -14,14 +14,20 @@ main = defaultMain tests
 
 tests :: TestTree
 tests = testGroup "Tests" [basic1,
-                           patternsOfPatterns
+                           parser1
+                           -- patternsOfPatterns
                           ]
 
 basic1 = testGroup "fast / slow"
          [
            testCase "silence" $ same16 (fast 1.1 silence) (silence :: Pattern Double),
            testCase "fast" $ same16  silence (silence :: Pattern Double),
-           testCase "fast" $ same16 "bd*128" (rep 128 "bd")
+           testCase "fast2" $ same16 "bd*128" (rep 128 "bd")
+         ]
+
+parser1 = testGroup "subpatterns"
+         [
+           testCase "square" $ same16 ("bd sn" :: Pattern String) ("[bd sn]" :: Pattern String)
          ]
 
 patternsOfPatterns =
